package fr.itsonus.bousoleplussbackend.security.jwt;

import fr.itsonus.bousoleplussbackend.security.SecurityConstants;
import io.jsonwebtoken.ExpiredJwtException;
import io.jsonwebtoken.Jwts;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.authentication.AuthenticationCredentialsNotFoundException;
import org.springframework.security.core.Authentication;
import org.springframework.stereotype.Component;

import java.util.Date;

@Component
public class JwtGenerator {

    @Value("${security.jwt.auth-expiration}")
    private int jwtAuthExpirationMs;

    @Value("${security.jwt.refresh-expiration}")
    private int jwtRefreshExpirationMs;

    @Value("${security.jwt.reset-password-link-expiration}")
    private int jwtResetPasswordLinkExpirationMs;

    public Token generateToken(String subject, long jwtExpirationMs) {
        var currentDate = new Date();
        var expiryDate = new Date(currentDate.getTime() + jwtExpirationMs);
        var token = Jwts.builder()
                .setSubject(subject)
                .setIssuedAt(currentDate)
                .setExpiration(expiryDate)
                .signWith(SecurityConstants.JWT_SECRET)
                .compact();
        return new Token("Bearer", token, currentDate.toInstant(), expiryDate.toInstant());
    }

    public Token generateResetPasswordToken(String subject) {
        return generateToken(subject, jwtResetPasswordLinkExpirationMs);
    }

    public Token generateAuthToken(Authentication authentication) {
        return generateToken(authentication.getName(), jwtAuthExpirationMs);
    }

    public Token generateRefreshToken(String subject) {
        return generateToken(subject, jwtRefreshExpirationMs);
    }

    public String getSubjectFromJWT(String token) {
        var builder = Jwts.parserBuilder()
                .setSigningKey(SecurityConstants.JWT_SECRET)
                .build();
        var claims = builder.parseClaimsJws(token)
                .getBody();
        return claims.getSubject();
    }

    public boolean validateToken(String token) {
        try {
            Jwts
                    .parserBuilder()
                    .setSigningKey(SecurityConstants.JWT_SECRET)
                    .build()
                    .parseClaimsJws(token);
            return true;
        } catch (ExpiredJwtException ex) {
            throw new AuthenticationCredentialsNotFoundException("JWT token is expired");
        } catch (Exception ex) {
            throw new AuthenticationCredentialsNotFoundException("JWT token is not valid");
        }
    }
}
